#include <stdio.h>
#include <stdlib.h>
/*
  LU Decomposion
*/
void LUDecomp(double** A, int n)
{
	int i,j,k;
	double dtemp;
	for(k=0;k<n;k++){
		dtemp=1.0/A[k][k];
		for(i=k+1;i<n;i++){
			A[i][k]*=dtemp;
		}
        	for(j=k+1;j<n;j++){
			dtemp=A[j][k];
            		for(i=k+1;i<n;i++){
				A[j][i]+=-A[k][i]*dtemp;
            		}
		}
	}
}
/*
  LU Solve
*/
void LUSolve(double** A, double *b, double *x, int n)
{
	int i,j;
	double sum;
	for(i=0;i<n;i++) x[i]=b[i];
	for(j=0;j<n-1;j++){
		for(i=j+1;i<n;i++){
            		x[i]-=A[i][j]*x[j];
        	}
    	}
	for(i=n-1;i>=0;i--){
		sum=0.0;
		for(j=i+1;j<n;j++){
			sum+=A[i][j]*x[j];
		}
		x[i]=(x[i]-sum)/A[i][i];
	}
}
/**************************
  allocate matrix (R)
**************************/
double** matrixR(int m,int n)
{
	double **aa;
	int i,j;
	if ( ( aa=(double ** )malloc( m * sizeof( double * ) ) ) == NULL ) {
		printf("Error:Memory is not enough! aa in matrix \n");
		exit(1);
	}
	if ( ( aa[0]=(double * )malloc( m * n * sizeof( double ) ) ) == NULL ) {
		printf("Error:Memory is not enough! aa in matrix \n");
		exit(1);
	}
	for(i=1;i<m;i++) aa[i]=aa[i-1]+n;
/*** zero clear ***/  
	for(j=0;j<n;j++){
		for(i=0;i<m;i++){
			aa[i][j]=0.0;
		}
	}
	return aa;
}
/**************************
  delete matrix (R)
**************************/
void free_matrixR(double **aa)
{
	free( (double *) aa[0] );
	free( (double **) aa );
}
/**************************
  allocate matrix (I)
**************************/
int** matrixI(int m,int n)
{
	int **aa;
	int i,j;
	if ( ( aa=(int ** )malloc( m * sizeof( int * ) ) ) == NULL ) {
		printf("Error:Memory is not enough! aa in matrix \n");
		exit(1);
	}
	if ( ( aa[0]=(int * )malloc( m * n * sizeof( int ) ) ) == NULL ) {
		printf("Error:Memory is not enough! aa in matrix \n");
		exit(1);
	}
	for(i=1;i<m;i++) aa[i]=aa[i-1]+n;
/*** zero clear ***/  
	for(j=0;j<n;j++){
		for(i=0;i<m;i++){
			aa[i][j]=0;
		}
	}
	return aa;
}
/**************************
  delete matrix (I)
**************************/
void free_matrixI(int **aa)
{
	free( (int *) aa[0] );
	free( (int **) aa );
}
/**************************
  allocate vector (R)
**************************/
double* vectorR(int m)
{
	double *a;
	int i;
	if ( ( a=(double * )malloc( m * sizeof( double ) ) ) == NULL ) {
	printf("Error:Memory is not enough! a in vector \n");
	exit(1);
}
/*** zero clear ***/  
	for(i=0;i<m;i++){
		a[i]=0.0;
	}
	return a;
}
/**************************
  delete vector (R)
**************************/
void free_vectorR(double *a)
{
	free( (double *) a );
}
/**************************
  allocate vector (I)
**************************/
int* vectorI(int m)
{
	int *a;
	int i;
	if ( ( a=(int * )malloc( m * sizeof( int ) ) ) == NULL ) {
	printf("Error:Memory is not enough! a in vector \n");
	exit(1);
}
/*** zero clear ***/  
	for(i=0;i<m;i++){
		a[i]=0;
	}
	return a;
}
/**************************
  delete vector (I)
**************************/
void free_vectorI(int *a)
{
	free( (int *) a );
}
