#include <stdio.h>
#include <stdlib.h>
/*
  LU Decomposion
*/
void LUDecomp(double** A, int n)
{
	int i,j,k;
	double dtemp;
	for(k=0;k<n;k++){
		dtemp=1.0/A[k][k];
		for(i=k+1;i<n;i++){
			A[i][k]*=dtemp;
		}
		for(j=k+1;j<n;j++){
			dtemp=A[j][k];
			for(i=k+1;i<n;i++){
				A[j][i]+=-A[k][i]*dtemp;
           	}
		}
	}
}
/*
  LU Solve
*/
void LUSolve(double** A, double *b, double *x,int n)
{
	int i,j;
	double sum;
	for(i=0;i<n;i++) x[i]=b[i];
	for(j=0;j<n-1;j++){
		for(i=j+1;i<n;i++){
			x[i]-=A[i][j]*x[j];
       	}
   	}
	for(i=n-1;i>=0;i--){
		sum=0.0;
		for(j=i+1;j<n;j++){
			sum+=A[i][j]*x[j];
		}
		x[i]=(x[i]-sum)/A[i][i];
	}
}
/**************************
  allocate matrix 
**************************/
double** matrixR(int m,int n)
{
	double **aa;
	int i,j;
	if ( ( aa=(double ** )malloc( m * sizeof( double * ) ) ) == NULL ) {
		printf("Error:Memory is not enough! aa in matrix \n");
		exit(1);
	}
	if ( ( aa[0]=(double * )malloc( m * n * sizeof( double ) ) ) == NULL ) {
		printf("Error:Memory is not enough! aa in matrix \n");
		exit(1);
	}
	for(i=1;i<m;i++) aa[i]=aa[i-1]+n;
/*** zero clear ***/  
	for(j=0;j<n;j++){
		for(i=0;i<m;i++){
			aa[i][j]=0.0;
		}
	}
	return aa;
}
/**************************
  delete matrix 
**************************/
void free_matrixR(double **aa)
{
	free( (double *) aa[0] );
	free( (double **) aa );
}
/**************************
  allocate vector
**************************/
double* vectorR(int m)
{
	double *a;
	int i;
	if ( ( a=(double * )malloc( m * sizeof( double ) ) ) == NULL ) {
	printf("Error:Memory is not enough! a in vector \n");
	exit(1);
}
/*** zero clear ***/  
	for(i=0;i<m;i++){
		a[i]=0.0;
	}
	return a;
}
/**************************
  delete vector
**************************/
void free_vectorR(double *a)
{
	free( (double *) a );
}
/*
 Main
*/
void main()
{
	int i,j,nn=3;
	double **a;
	double *b,*x;
/* allocate matrix */
	a=matrixR(nn,nn);
/* set matrix */
	a[0][0]=2;a[0][1]=3;a[0][2]=1;
	a[1][0]=2;a[1][1]=1;a[1][2]=-2;
	a[2][0]=1;a[2][1]=2;a[2][2]=3;
	printf("MATRIX A\n");
	for(i=0;i<nn;i++){
		for(j=0;j<nn;j++){
			printf("%e ",a[i][j]);
		}
		printf("\n");
	}
/* set vector */
	b=vectorR(nn);
	x=vectorR(nn);
	b[0]=5;
	b[1]=1;
	b[2]=7;
/* LU decomposition */
	LUDecomp(a,nn);
	printf("LU of A\n");
	for(i=0;i<nn;i++){
		for(j=0;j<nn;j++){
			printf("%e ",a[i][j]);
		}
		printf("\n");
	}
/* Solve */
	LUSolve(a,b,x,nn);
	for(i=0;i<nn;i++){
		printf("%d %e\n",i,x[i]);
	}
/* Freeing */
	free_matrixR(a);
	free_vectorR(b);
	free_vectorR(x);
}
