#include <stdio.h>
#include <stdlib.h>
#include "LIB.h"
/*
    define NN : Number of nodes per element
*/
#define NN 2
/*
    stiffness :: k_bar1d2
*/
void k_bar1d2(double node[],double ee,double aa,double stf[][2])
{
	double length;
	length=node[1]-node[0];
	stf[0][0]= ee*aa/length;
	stf[0][1]=-ee*aa/length;
	stf[1][0]=-ee*aa/length;
	stf[1][1]= ee*aa/length;
}
/*
   stress recovery:: stress_bar1d2
*/
double  stress_bar1d2(double node[],double ee,double disp[])
{
	double length,sgm;
	length=node[1]-node[0];
	sgm=ee/length*(-disp[0]+disp[1]);
	return sgm;
}
/*
   body force :: bf_bar1d2
*/
void bf_bar1d2(double node[],double aa,double load,double bf[])
{
	double length;
	length=node[1]-node[0];
	bf[0]=load*aa*length*0.5;
	bf[1]=load*aa*length*0.5;
}
void main()
{
	int i,j,k,ie,count,irow,icol,npos;
/* for FEM model **/
	int numnp,nume,nspc,ncload,ndload;
	double *node;
	int    **element;
	int    *spc;
	double E,A;
	int    *cload;
	double *vcload;
	int    *dload;
	double *vdload;
/* for Local system **/
	double nodeL[2];
	double kL[2][2];
	double dispL[2];
	double bf[4];
	double f;
/* for Global system **/
	double **KG,**row,*FC,*FD,*F,*U;
/*
    Node definitions:
*/
/** number of divisions **/
#define NDIV 20
	numnp=NDIV+1;
	node=vectorR(numnp);
	for(i=0;i<numnp;i++) node[i]=i/(double)(NDIV);
	printf("///node :: numnp=%d\n",numnp);
	for (i=0;i<numnp;i++){
		printf("%d %e\n",i,node[i]);
	}
/*
    Element definitions:
*/
	nume=NDIV;
	element=matrixI(nume,2);
	for(i=0;i<nume;i++) {
		element[i][0]=i+1;
		element[i][1]=i+2;
	}
	printf("///element :: nume=%d\n",nume);
	for (i=0;i<nume;i++){
		for (j=0;j<NN;j++){
			printf("%d ",element[i][j] );
		}
		printf("\n");
	}
/*
    Material definitions:
*/
	E=1.0;
	A=1.0;
	printf("///material :: E=%e, A=%e\n",E,A);
/*
    SPC definitions:
*/
	nspc=1;
	spc=vectorI(nspc);
	spc[0]=numnp;
	printf("///spc :: nspc=%d\n",nspc);
	for (i=0;i<nspc;i++){
		printf("%d %d\n",i+1,spc[i]);
	}
	row=matrixR(nspc,numnp);
/* 
  load definitions:
*/
	FC=vectorR(numnp);
	FD=vectorR(numnp);
	F =vectorR(numnp);
	U =vectorR(numnp);
	ncload=0;
	if( ncload > 0 ){
		cload=vectorI(ncload);
		vcload=vectorR(ncload);
	}	
	printf("///cload :: ncload=%d\n",ncload);
	for (i=0;i<ncload;i++){
		printf("%d %d %e\n",i+1,cload[i],vcload[i]);
	}
	ndload=nume;
	if( ndload > 0 ){
		dload=vectorI(ndload);
		vdload=vectorR(ndload);
	}
	for (i=0;i<ndload;i++){
		dload[i]=i+1;vdload[i]=-1.0;
	}
	printf("///dload :: ndload=%d\n",ndload);
	for (i=0;i<ndload;i++){
		printf("%d %d %e\n",i+1,dload[i],vdload[i]);
	}
/*
	Global stiffness numnp x numnp
*/
	KG=matrixR(numnp,numnp);
/*
    Loop over elements
*/
	for (ie=0;ie<nume;ie++){
/* Local element coordinate */
		for (i=0;i<NN;i++){
			nodeL[i]=node[element[ie][i]-1];
		}
/* Local elememt stiffness */
		k_bar1d2(nodeL,E,A,kL);
/* Add element 1 to Global stiffness */ 
		for (i=0;i<NN;i++){
			irow=element[ie][i];
			for (j=0;j<NN;j++){
				icol=element[ie][j];
				KG[irow-1][icol-1]+=kL[i][j];
			}
		}
	}
/*
	Load vector (concentrated load)
*/
	for(i=0;i<numnp;i++){
		FC[i]=0.0;
	}
	for (i=0;i<ncload;i++){
		npos=cload[i];
		FC[npos-1]+=vcload[i];
	}
/*
    Load vector (distributed load)
*/
	for(i=0;i<numnp;i++){
		FD[i]=0.0;
	}
	for (i=0;i<ndload;i++){
		ie=dload[i];
		for (j=0;j<NN;j++){
			nodeL[j]=node[element[ie-1][j]-1];
		}
		bf_bar1d2(nodeL,A,vdload[i],bf);
		for (j=0;j<NN;j++){
			irow=element[ie-1][j];
			FD[irow-1]+=bf[j];
		}
	}
/*
    TOTAL LOAD
*/
	for(i=0;i<numnp;i++){
		F[i]=FC[i]+FD[i];
	}
/*
    constrained conditions
*/
	count=1;
	for (i=0;i<nspc;i++){
		npos=spc[i];
		for(j=0;j<numnp;j++){
			row[count-1][j]=KG[npos-1][j];
			KG[npos-1][j]=0.0;KG[j][npos-1];KG[npos-1][npos-1]=1;
			F[npos-1]=0.0;
		}
		count++;
	}
/*
    computation 
*/
	printf("### Computation started\n");
	LUDecomp(KG,numnp);
	LUSolve(KG,F,U,numnp);
	printf("### Computation finished\n");
	printf("### Nodal displacement\n");
	for (i=0;i<numnp;i++){
		printf("%d %e\n",i+1,U[i]);
	}
/* post processing :: stress */
	printf("### Element stress\n");
	for (ie=0;ie<nume;ie++){
		for (j=0;j<NN;j++){
			nodeL[j]=node[element[ie][j]-1];
			dispL[j]=U[element[ie][j]-1];
		}
		printf("%d %e\n",ie+1,stress_bar1d2(nodeL,E,dispL));
	}
/* post processing :: reaction force */
	printf("### Reaction force\n");
	count=1;
	for (i=0;i<nspc;i++){
		f=-FD[spc[i]-1];
		for (k=0;k<numnp;k++){
			f+=row[count-1][k]*U[k];
		}
		count++;
		printf("%d %e\n",spc[i],f);
	}
/* freeing */
	free_vectorR(node);
	free_matrixI(element);
	free_vectorI(spc);
	if( ncload > 0 ) free_vectorI(cload);
	if( ncload > 0 ) free_vectorR(vcload);
	if( ndload > 0 ) free_vectorI(dload);
	if( ndload > 0 ) free_vectorR(vdload);
	free_matrixR(KG);
	free_matrixR(row);
	free_vectorR(FC);
	free_vectorR(FD);
	free_vectorR(F);
	free_vectorR(U);
/*
   finished
*/
}
