%%% truss22vtf.m : ver2018.2.20
clear all;
%%%
%%% NDIM definitions: Dimension of problem
%%%
NDIM=2;
%%%
%%% NDOF definitions: Number of DOFs per node
%%%
NDOF=2;
%%%
%%% NN definitions: Number of nodes per element
%%%
NN=2;
%%%
%%% stiffeness :: k_truss2d2
%%%
function stf=k_truss2d2(node,ee,aa)
	v(1)=node(2,1)-node(1,1);
	v(2)=node(2,2)-node(1,2);
	length=sqrt( v(1)^2 + v(2)^2 );
	c=v(1)/length;
	s=v(2)/length;
	k=ee*aa/length;
	stf(1,1)= k*c*c;
	stf(1,2)= k*c*s;
	stf(1,3)=-k*c*c;
	stf(1,4)=-k*s*c;
	stf(2,1)= k*c*s;
	stf(2,2)= k*s*s;
	stf(2,3)=-k*s*c;
	stf(2,4)=-k*s*s;
	stf(3,1)=-k*c*c;
	stf(3,2)=-k*c*s;
	stf(3,3)= k*c*c;
	stf(3,4)= k*c*s;
	stf(4,1)=-k*c*s;
	stf(4,2)=-k*s*s;
	stf(4,3)= k*c*s;
	stf(4,4)= k*s*s;
end
%%%
%%% internal force recovery :: s_trusst2d2
%%%
function force=s_truss2d2(node,ee,aa,disp)
	v(1)=node(2,1)-node(1,1);
	v(2)=node(2,2)-node(1,2);
	length=sqrt( v(1)^2 + v(2)^2 );
	c=v(1)/length;
	s=v(2)/length;
	force=ee*aa/length*(-c*disp(1,1)-s*disp(1,2)+c*disp(2,1)+s*disp(2,2));
end
%%%
%%% body force :: bf_truss2d2
%%%
function bf=bf_truss2d2(node,aa,load)
	v(1)=node(2,1)-node(1,1);
	v(2)=node(2,2)-node(1,2);
	length=sqrt( v(1)^2 + v(2)^2 );
	bf(1)=0.5*aa*length*load(1);
	bf(2)=0.5*aa*length*load(2);
	bf(3)=0.5*aa*length*load(1);
	bf(4)=0.5*aa*length*load(2);
end
%%%
%%% VTK :: writeVTK
%%%
function writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,e_stress)
%%% VTK_CELL_TYPE 
	vtk_cell_type=3;
%%% NVARS
	NVARS=1; 
	vname=char("S11");
%%% Header 
	fprintf(fid,"# vtk DataFile Version 2.0\n");
	fprintf(fid,"%s\n",program_name());
	fprintf(fid,"ASCII\n");
%%% Node 
	fprintf(fid,"DATASET UNSTRUCTURED_GRID\n");
	fprintf(fid,"POINTS %d float\n",numnp);
	for i=1:numnp
		xc(1)=0.0;xc(2)=0.0;xc(3)=0.0;
		for j=1:NDIM
			xc(j)=node(i,j);
		end
		fprintf(fid,"%e %e %e\n",xc(1),xc(2),xc(3));
	end
%%% Element
	fprintf(fid,"CELLS %d %d\n",nume,nume*(1+NN));
	for i=1:nume
		for j=1:NN
			nid(j)=element(i,j);
		end
		fprintf(fid,"%d ",NN);
		for j=1:NN
			fprintf(fid,"%d ",nid(j)-1);
		end
		fprintf(fid,"\n");
	end
%%% type
	fprintf(fid,"CELL_TYPES %d\n",nume);
for i=1:nume
	fprintf(fid,"%d\n",vtk_cell_type);
end
%%% Result for NODE
	fprintf(fid,"POINT_DATA %d\n",numnp);
%%% Deformation 
	fprintf(fid,"VECTORS DISPLACEMENT float\n");
	for i=1:numnp
		if( NDIM == 2)
			fprintf(fid," %e %e 0.0\n",U(2*i-1),U(2*i));
		endif
	end
%%% Result for ELEMENT
	fprintf(fid,"CELL_DATA %d\n",nume);
%%% Element Stress 
	for i=1:NVARS
		fprintf(fid,"SCALARS %s float 1\n",vname(i,:));
		fprintf(fid,"LOOKUP_TABLE default\n");
		for j=1:nume
			fprintf(fid,"%e\n",e_stress(j));
		end
	end
end
%%%
%%% Node definitions:
%%%
numnp=12;
node(1,:)=[0.     0.0];
node(2,:)=[3000.  0.0];
node(3,:)=[6000.  0.0];
node(4,:)=[9000.  0.0];
node(5,:)=[12000. 0.0];
node(6,:)=[15000. 0.0];
node(7,:)=[0.     4000.0];
node(8,:)=[3000.  4000.0];
node(9,:)=[6000.  4000.0];
node(10,:)=[9000.  4000.0];
node(11,:)=[12000. 4000.0];
node(12,:)=[15000. 4000.0];
printf("///node :: numnp=%d\n",numnp);
for i=1:numnp
	printf("%d ",i);
	for j=1:NDIM
		printf("%e ",node(i,j));
	end
	printf("\n");
end
%%%
%%% Element definitions:
%%%
nume=22;
element(1,:)=[ 1    2];
element(2,:)=[ 2    3];
element(3,:)=[ 3    4];
element(4,:)=[ 4    5];
element(5,:)=[ 5    6];
element(6,:)=[ 7    8];
element(7,:)=[ 8    9];
element(8,:)=[ 9   10];
element(9,:)=[10   11];
element(10,:)=[11   12];
element(11,:)=[ 1    7];
element(12,:)=[ 2    8];
element(13,:)=[ 3    9];
element(14,:)=[ 4   10];
element(15,:)=[ 5   11];
element(16,:)=[ 6   12];
element(17,:)=[ 2    7];
element(18,:)=[ 3    8];
element(19,:)=[ 4    9];
element(20,:)=[ 3   10];
element(21,:)=[ 4   11];
element(22,:)=[ 5   12];
printf("///element :: nume=%d\n",nume);
for i=1:nume
	printf("%d ",i);
	for j=1:NN
		printf("%d ",element(i,j) );
	end
	printf("\n");
end
%%%
%%% Material definitions:
%%%
E=100.0e3;
A=10000.0;
printf("///material :: E=%e, A=%e\n",E,A);
%%%
%%% SPC definitions:
%%%
nspc=4;
spc(1,:)=[1 1];
spc(2,:)=[1 2];
spc(3,:)=[6 1];
spc(4,:)=[6 2];
printf("///spc :: nspc=%d\n",nspc);
for i=1:nspc
	printf("%d %d %d\n",i,spc(i,1),spc(i,2))
end
%%%
%%% load definitions:
%%%
ncload=0;
cload(1,:)=[1 2];vcload(1)=-1.0;
printf("///cload :: ncload=%d\n",ncload);
for i=1:ncload
	printf("%d %d %d %e\n",i,cload(i,1),cload(i,2),vcload(i))
end
ndload=nume;
for i=1:ndload
	dload(i)=i;
	vdload(i,:)=[0.0 -78.4e-6];
end
printf("///dload :: ndload=%d\n",ndload);
for i=1:ndload
	printf("%d %d %e %e\n",i,dload(i),vdload(i,1),vdload(i,2))
end
%%%
%%% Global stiffness (NDOF*numnp) x (NDOF*numnp)
%%%
KG=zeros(NDOF*numnp,NDOF*numnp);
%%%
%%% Loop over elements
%%%
for ie=1:nume
%%% Local element coordinate
	for i=1:NN
		for j=1:NDIM
			nodeL(i,j)=node(element(ie,i),j);
		end
	end
%%% Local elememt stiffness element
	kL=k_truss2d2(nodeL,E,A);
%%% Add element 1 to Global stiffness 
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			for j=1:NN
				for jj=1:NDOF
					icol=NDOF*(element(ie,j)-1)+jj;
					KG(irow,icol)=KG(irow,icol)+kL(NDOF*(i-1)+ii,NDOF*(j-1)+jj);
				end
			end
		end
	end
end
%%%
%%% Load vector (condentrated load)
%%%
FC=zeros(NDOF*numnp,1);
for i=1:ncload
	npos=NDOF*(cload(i,1)-1)+cload(i,2)
	FC(npos)=FC(npos)+vcload(i);
end
%%%
%%% Load vector (distributed load)
%%%
FD=zeros(NDOF*numnp,1);
for i=1:ndload
	ie=dload(i);
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
	end
	bf=bf_truss2d2(nodeL,A,vdload(i,:));
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			FD(irow)=FD(irow)+bf(NDOF*(i-1)+ii);
		end
	end
endfor
%%%
%%% TOTAL LOAD
%%%
	F=FC+FD;
%%%
%%% constrained conditions
%%%
count=1;
for i=1:nspc
	npos=NDOF*(spc(i,1)-1)+spc(i,2);
	row(count,:)=KG(npos,:);
	KG(npos,:)=0;
	KG(:,npos)=0;
	KG(npos,npos)=1;
	F(npos)=0.0;
	count=count+1;
end
%%%
%%% computation 
%%%
printf("### Computation started\n");
U=KG\F;
printf("### Computation finished\n");
printf("### Result displacement\n")
for i=1:numnp
	printf("%d ",i);
	for j=1:NDOF
		printf("%e ",U(NDOF*(i-1)+j));
	end
	printf("\n");
end
%%% post processing :: stress
%%% for element stress/S11
e_stress=zeros(nume);
printf("Element force\n")
for ie=1:nume
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
		for k=1:NDOF
			dispL(j,k)=U(NDOF*(element(ie,j)-1)+k);
		end
	end
	s11=s_truss2d2(nodeL,E,A,dispL);
	printf("%d %e\n",ie,s11);
	e_stress(ie)=s11;
end
%%% post processing :: reaction force
printf("Reaction force\n")
count=1;
for i=1:nspc
	f=-FD(NDOF*(spc(i,1)-1)+spc(i,2));
	for k=1:NDOF*numnp
		f=f+row(count,k)*U(k);
	end
	count=count+1;
	printf("%d %d %e\n",spc(i,1),spc(i,2),f);
end
%%%
%%% for visualization
%%%
	fid=fopen('vtk.vtk','w');
	writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,e_stress);
	fclose(fid);
%%%
%%% finished
%%%
