%%% truss3.m :: ver2018.2.20
clear all;
%%%
%%% NDIM definitions: Dimension of problem
%%%
NDIM=2;
%%%
%%% NDOF definitions: Number of DOFs per node
%%%
NDOF=2;
%%%
%%% NN definitions: Number of nodes per element
%%%
NN=2;
%%%
%%% stiffeness :: k_truss2d2
%%%
function stf=k_truss2d2(node,ee,aa)
	v(1)=node(2,1)-node(1,1);
	v(2)=node(2,2)-node(1,2);
	length=sqrt( v(1)^2 + v(2)^2 );
	c=v(1)/length;
	s=v(2)/length;
	k=ee*aa/length;
	stf(1,1)= k*c*c;
	stf(1,2)= k*c*s;
	stf(1,3)=-k*c*c;
	stf(1,4)=-k*s*c;
	stf(2,1)= k*c*s;
	stf(2,2)= k*s*s;
	stf(2,3)=-k*s*c;
	stf(2,4)=-k*s*s;
	stf(3,1)=-k*c*c;
	stf(3,2)=-k*c*s;
	stf(3,3)= k*c*c;
	stf(3,4)= k*c*s;
	stf(4,1)=-k*c*s;
	stf(4,2)=-k*s*s;
	stf(4,3)= k*c*s;
	stf(4,4)= k*s*s;
end
%%%
%%% internal force recovery :: s_trusst2d2
%%%
function force=s_truss2d2(node,ee,aa,disp)
	v(1)=node(2,1)-node(1,1);
	v(2)=node(2,2)-node(1,2);
	length=sqrt( v(1)^2 + v(2)^2 );
	c=v(1)/length;
	s=v(2)/length;
	force=ee*aa/length*(-c*disp(1,1)-s*disp(1,2)+c*disp(2,1)+s*disp(2,2));
end
%%%
%%% body force :: bf_truss2d2
%%%
function bf=bf_truss2d2(node,aa,load)
	v(1)=node(2,1)-node(1,1);
	v(2)=node(2,2)-node(1,2);
	length=sqrt( v(1)^2 + v(2)^2 );
	bf(1)=0.5*aa*length*load(1);
	bf(2)=0.5*aa*length*load(2);
	bf(3)=0.5*aa*length*load(1);
	bf(4)=0.5*aa*length*load(2);
end
%%%
%%% Node definitions:
%%%
numnp=4;
node(1,:)=[ 0.   0.0];
node(2,:)=[-3/2  sqrt(3)/2];
node(3,:)=[ 1/2  sqrt(3)/2];
node(4,:)=[ 0.   sqrt(3)/2];
printf("///node :: numnp=%d\n",numnp);
for i=1:numnp
	printf("%d ",i);
	for j=1:NDIM
		printf("%e ",node(i,j));
	end
	printf("\n");
end
%%%
%%% Element definitions:
%%%
nume=3;
element(1,:)=[ 1    2];
element(2,:)=[ 1    3];
element(3,:)=[ 1    4];
printf("///element :: nume=%d\n",nume);
for i=1:nume
	printf("%d ",i);
	for j=1:NN
		printf("%d ",element(i,j) );
	end
	printf("\n");
end
%%%
%%% Material definitions:
%%%
E=1.0;
A=1.0;
printf("///material :: E=%e, A=%e\n",E,A);
%%%
%%% SPC definitions:
%%%
nspc=6;
spc(1,:)=[2 1];
spc(2,:)=[2 2];
spc(3,:)=[3 1];
spc(4,:)=[3 2];
spc(5,:)=[4 1];
spc(6,:)=[4 2];
printf("///spc :: nspc=%d\n",nspc);
for i=1:nspc
	printf("%d %d %d\n",i,spc(i,1),spc(i,2))
end
%%%
%%% load definitions:
%%%
ncload=1;
cload(1,:)=[1 2];vcload(1)=-1.0;
printf("///cload :: ncload=%d\n",ncload);
for i=1:ncload
	printf("%d %d %d %e\n",i,cload(i,1),cload(i,2),vcload(i))
end
ndload=0;
for i=1:ndload
	dload(i)=i;
	vdload(i,:)=[0.0 -1.0];
end
printf("///dload :: ndload=%d\n",ndload);
for i=1:ndload
	printf("%d %d %e %e\n",i,dload(i),vdload(i,1),vdload(i,2))
end
%%%
%%% Global stiffness (NDOF*numnp) x (NDOF*numnp)
%%%
KG=zeros(NDOF*numnp,NDOF*numnp);
%%%
%%% Loop over elements
%%%
for ie=1:nume
%%% Local element coordinate
	for i=1:NN
		for j=1:NDIM
			nodeL(i,j)=node(element(ie,i),j);
		end
	end
%%% Local elememt stiffness element
	kL=k_truss2d2(nodeL,E,A);
%%% Add element 1 to Global stiffness 
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			for j=1:NN
				for jj=1:NDOF
					icol=NDOF*(element(ie,j)-1)+jj;
					KG(irow,icol)=KG(irow,icol)+kL(NDOF*(i-1)+ii,NDOF*(j-1)+jj);
				end
			end
		end
	end
end
%%%
%%% Load vector (condentrated load)
%%%
FC=zeros(NDOF*numnp,1);
for i=1:ncload
	npos=NDOF*(cload(i,1)-1)+cload(i,2)
	FC(npos)=FC(npos)+vcload(i);
end
%%%
%%% Load vector (distributed load)
%%%
FD=zeros(NDOF*numnp,1);
for i=1:ndload
	ie=dload(i);
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
	end
	bf=bf_truss2d2(nodeL,A,vdload(i,:));
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			FD(irow)=FD(irow)+bf(NDOF*(i-1)+ii);
		end
	end
end
%%%
%%% TOTAL LOAD
%%%
	F=FC+FD;
%%%
%%% constrained conditions
%%%
count=1
for i=1:nspc
	npos=NDOF*(spc(i,1)-1)+spc(i,2);
	row(count,:)=KG(npos,:);
	KG(npos,:)=0;
	KG(:,npos)=0;
	KG(npos,npos)=1;
	F(npos)=0.0;
	count=count+1;
end
%%%
%%% computation 
%%%
printf("### Computation started\n");
U=KG\F;
printf("### Computation finished\n");
printf("### Nodal displacement\n")
for i=1:numnp
	printf("%d ",i);
	for j=1:NDOF
		printf("%e ",U(NDOF*(i-1)+j));
	end
	printf("\n");
end
%%% post processing :: stress
printf("### Element force\n")
for ie=1:nume
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
		for k=1:NDOF
			dispL(j,k)=U(NDOF*(element(ie,j)-1)+k);
		end
	end
	printf("%d %e\n",ie,s_truss2d2(nodeL,E,A,dispL));
end
%%% post processing :: reaction force
printf("### Reaction force\n")
count=1;
for i=1:nspc
	f=-FD(NDOF*(spc(i,1)-1)+spc(i,2));
	for k=1:NDOF*numnp
		f=f+row(count,k)*U(k);
	endfor
	count=count+1;
	printf("%d %d %e\n",spc(i,1),spc(i,2),f);
end
%%%
%%% finished
%%%
