%%% quadFEM.m :: ver2018.2.20
clear all;
%%%
%%% NDIM definitions: Dimension of problem
%%%
NDIM=2;
%%%
%%% NDOF definitions: Number of DOFs per element
%%%
NDOF=2;
%%%
%%% NN definitions: Number of nodes per element
%%%
NN=4;
%%%
%%% stiffeness :: k_quad2d4
%%%
function stf=k_quad2d4(node,type,ee,poi)
%%% set [D]
if( type == 0 )
	coef1=ee/((1.0+poi)*(1.0-2.0*poi));
	coef2=ee/(2.0*(1.0+poi));
	D(1,1)=coef1*(1.0-poi);
	D(1,2)=coef1*poi;
	D(1,3)=0.0;
	D(2,1)=coef1*poi;
	D(2,2)=coef1*(1.0-poi);
	D(2,3)=0.0;
	D(3,1)=0.0;
	D(3,2)=0.0;
	D(3,3)=coef2;
else
	coef1=ee/(1-poi*poi);
	coef2=0.5*(1-poi);
	D(1,1)=coef1;
	D(1,2)=coef1*poi;
	D(1,3)=0.0;
	D(2,1)=D(1,2);
	D(2,2)=D(1,1);
	D(2,3)=0.0;
	D(3,1)=0.0;
	D(3,2)=0.0;
	D(3,3)=coef1*coef2;
end
%%%
stf=zeros(8,8);
B=zeros(3,8);
%%% integration
xg=[-1/sqrt(3), 1/sqrt(3)];
wg=[1,1];
for i=1:2
	r1=xg(i);
	for j=1:2
		r2=xg(j);
%%% interpolation function
		r1p=1+r1;
		r2p=1+r2;
		r1m=1-r1;
		r2m=1-r2;
		N(1)=r1m*r2m/4;
		N(2)=r1p*r2m/4;
		N(3)=r1p*r2p/4;
		N(4)=r1m*r2p/4;
%%% derivative of interpolation function  for r1
		Nr1(1)=-r2m/4;
		Nr1(2)= r2m/4;
		Nr1(3)= r2p/4;
		Nr1(4)=-r2p/4;
%%% derivative of interpolation function  for r2
		Nr2(1)=-r1m/4;
		Nr2(2)=-r1p/4;
		Nr2(3)= r1p/4;
		Nr2(4)= r1m/4;
%%% Jacobi matrix 
		J=zeros(2,2);
		for k=1:4
			J(1,1)=J(1,1)+Nr1(k)*node(k,1);
			J(1,2)=J(1,2)+Nr1(k)*node(k,2);
			J(2,1)=J(2,1)+Nr2(k)*node(k,1);
			J(2,2)=J(2,2)+Nr2(k)*node(k,2);
		end
%%% inverse of Jacobian
		Ji=inv(J);
%%% B matrix
		for k=1:4
			dndx=Ji(1,1)*Nr1(k)+Ji(1,2)*Nr2(k);
			dndy=Ji(2,1)*Nr1(k)+Ji(2,2)*Nr2(k);
			B(1,2*k-1)=dndx;
			B(2,2*k-1)=0.0;
			B(3,2*k-1)=dndy;
			B(1,2*k  )=0;
			B(2,2*k  )=dndy;
			B(3,2*k  )=dndx;
		end
%%% stiffness
		stf=stf+B'*D*B*det(J)*wg(i)*wg(j);
	end
end
end
%%%
%%%  stress recovery :: rcv_quad2d4
%%%
function [s e]=rcv_quad2d4(node,type,ee,poi,disp)
%%% set [D]
if( type == 0 )
	coef1=ee/((1.0+poi)*(1.0-2.0*poi));
	coef2=ee/(2.0*(1.0+poi));
	D(1,1)=coef1*(1.0-poi);
	D(1,2)=coef1*poi;
	D(1,3)=0.0;
	D(2,1)=coef1*poi;
	D(2,2)=coef1*(1.0-poi);
	D(2,3)=0.0;
	D(3,1)=0.0;
	D(3,2)=0.0;
	D(3,3)=coef2;
else
	coef1=ee/(1-poi*poi);
	coef2=0.5*(1-poi);
	D(1,1)=coef1;
	D(1,2)=coef1*poi;
	D(1,3)=0.0;
	D(2,1)=D(1,2);
	D(2,2)=D(1,1);
	D(2,3)=0.0;
	D(3,1)=0.0;
	D(3,2)=0.0;
	D(3,3)=coef1*coef2;
end
%%% Local displacement
dd(1)=disp(1,1);
dd(2)=disp(1,2);
dd(3)=disp(2,1);
dd(4)=disp(2,2);
dd(5)=disp(3,1);
dd(6)=disp(3,2);
dd(7)=disp(4,1);
dd(8)=disp(4,2);
%%%
B=zeros(3,8);
%%% integration
xg=[-1/sqrt(3), 1/sqrt(3)];
count=1;
for i=1:2
	r1=xg(i);
	for j=1:2
		r2=xg(j);
%%% interpolation function
		r1p=1+r1;
		r2p=1+r2;
		r1m=1-r1;
		r2m=1-r2;
		N(1)=r1m*r2m/4;
		N(2)=r1p*r2m/4;
		N(3)=r1p*r2p/4;
		N(4)=r1m*r2p/4;
%%% derivative of interpolation function  for r1
		Nr1(1)=-r2m/4;
		Nr1(2)= r2m/4;
		Nr1(3)= r2p/4;
		Nr1(4)=-r2p/4;
%%% derivative of interpolation function  for r2
		Nr2(1)=-r1m/4;
		Nr2(2)=-r1p/4;
		Nr2(3)= r1p/4;
		Nr2(4)= r1m/4;
%%% Jacobi matrix 
		J=zeros(2,2);
		for k=1:4
			J(1,1)+=Nr1(k)*node(k,1);
			J(1,2)+=Nr1(k)*node(k,2);
			J(2,1)+=Nr2(k)*node(k,1);
			J(2,2)+=Nr2(k)*node(k,2);
		end
%%% inverse of Jacobian
		Ji=inv(J);
%%% B matrix
		for k=1:4
			dndx=Ji(1,1)*Nr1(k)+Ji(1,2)*Nr2(k);
			dndy=Ji(2,1)*Nr1(k)+Ji(2,2)*Nr2(k);
			B(1,2*k-1)=dndx;
			B(2,2*k-1)=0.0;
			B(3,2*k-1)=dndy;
			B(1,2*k  )=0;
			B(2,2*k  )=dndy;
			B(3,2*k  )=dndx;
		end
%%% stress & strain
		strain=B*dd';
		stress=D*strain;
		for k=1:3
			s(count,k)=stress(k);
			e(count,k)=strain(k);
		end
%%% compute S33
		s(count,4)=0.0;
		if( type == 0 )
			s(count,4)=poi*(s(count,1)+s(count,2));
		end
		count=count+1;
	end
end
end
%%%
%%% distributed load :: dload_quad2d4
%%%
function vect=dload_quad2d4(node,ltype,val)
%%% ltype=1  BX: BODY FORCE IN X-DIRECTION
%%% ltype=2  BY: BODY FORCE IN Y-DIRECTION
%%% ltype=10 P1: TRACTION IN NORMAL-DIRECTION FOR FACE-1
%%% ltype=20 P2: TRACTION IN NORMAL-DIRECTION FOR FACE-2
%%% ltype=30 P3: TRACTION IN NORMAL-DIRECTION FOR FACE-3
%%% ltype=40 P4: TRACTION IN NORMAL-DIRECTION FOR FACE-4
	ivol=0;
	isuf=0;
	if( ltype < 10 )
		ivol=1;
	else
		isuf=1;
	end
%%% CLEAR
	vect=zeros(8,1);
%%% SURFACE LOAD
	if( isuf == 1 )
		switch( ltype )
			case (10)
			nod1=1;nod2=2;
			case (20)
			nod1=2;nod2=3;
			case (30)
			nod1=3;nod2=4;
			case (40)
			nod1=4;nod2=1;
		end
		px=val*(node(nod1,2)-node(nod2,2));
		py=val*(node(nod2,1)-node(nod1,1));
		vect(2*nod1-1)=px*0.5;
		vect(2*nod1  )=py*0.5;
		vect(2*nod2-1)=px*0.5;
		vect(2*nod2  )=py*0.5;
	end
%%% VOLUME LOAD
	if( ivol == 1 )
%%% integration
		xg=[-1/sqrt(3), 1/sqrt(3)];
		wg=[1,1];
		for i=1:2
			r1=xg(i);
			for j=1:2
				r2=xg(j);
%%% interpolation function
				r1p=1+r1;
				r2p=1+r2;
				r1m=1-r1;
				r2m=1-r2;
				N(1)=r1m*r2m/4;
				N(2)=r1p*r2m/4;
				N(3)=r1p*r2p/4;
				N(4)=r1m*r2p/4;
%%% derivative of interpolation function  for ri
				Nr1(1)=-r2m/4;
				Nr1(2)= r2m/4;
				Nr1(3)= r2p/4;
				Nr1(4)=-r2p/4;
%%% derivative of interpolation function  for si
				Nr2(1)=-r1m/4;
				Nr2(2)=-r1p/4;
				Nr2(3)= r1p/4;
				Nr2(4)= r1m/4;
%%% Jacobi matrix 
				J=zeros(2,2);
				for k=1:4
					J(1,1)=J(1,1)+Nr1(k)*xx(k);
					J(1,2)=J(1,2)+Nr1(k)*yy(k);
					J(2,1)=J(2,1)+Nr2(k)*xx(k);
					J(2,2)=J(2,2)+Nr2(k)*yy(k);
				end
				wgt=wg(i)*wg(j)*det(J);
%%% load
				switch( ltype )
					case (10)
					vect(1)=vect(1)+N(1)*wgt*val;
					vect(3)=vect(3)+N(2)*wgt*val;
					vect(5)=vect(5)+N(3)*wgt*val;
					vect(7)=vect(7)+N(4)*wgt*val;
					case (20)
					vect(2)=vect(2)+N(1)*wgt*val;
					vect(4)=vect(4)+N(2)*wgt*val;
					vect(6)=vect(6)+N(3)*wgt*val;
					vect(8)=vect(8)+N(4)*wgt*val;
				end
			end
		end
	end
end
%%%
%%% VTK :: writeVTK
%%%
function writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,n_stress)
%%% VTK_CELL_TYPE 
	vtk_cell_type=9;
%%% NVARS
	NVARS=4; 
	vname=char("S11","S22","S12","S33");
%%% Header 
	fprintf(fid,"# vtk DataFile Version 2.0\n");
	fprintf(fid,"%s\n",program_name());
	fprintf(fid,"ASCII\n");
%%% Node 
	fprintf(fid,"DATASET UNSTRUCTURED_GRID\n");
	fprintf(fid,"POINTS %d float\n",numnp);
	for i=1:numnp
		xc(1)=0.0;xc(2)=0.0;xc(3)=0.0;
		for j=1:NDIM
			xc(j)=node(i,j);
		end
		fprintf(fid,"%e %e %e\n",xc(1),xc(2),xc(3));
	end
%%% Element
	fprintf(fid,"CELLS %d %d\n",nume,nume*(1+NN));
	for i=1:nume
		for j=1:NN
			nid(j)=element(i,j);
		end
		fprintf(fid,"%d ",NN);
		for j=1:NN
			fprintf(fid,"%d ",nid(j)-1);
		end
		fprintf(fid,"\n");
	end
%%% type
	fprintf(fid,"CELL_TYPES %d\n",nume);
	for i=1:nume
		fprintf(fid,"%d\n",vtk_cell_type);
	end
%%% Result 
	fprintf(fid,"POINT_DATA %d\n",numnp);
%%% Deformation 
	fprintf(fid,"VECTORS DISPLACEMENT float\n");
	for i=1:numnp
		if( NDIM == 2)
			fprintf(fid," %e %e 0.0\n",U(2*i-1),U(2*i));
		end
	end
%%% Nodal Stress 
	for i=1:NVARS
		fprintf(fid,"SCALARS %s float 1\n",vname(i,:));
		fprintf(fid,"LOOKUP_TABLE default\n");
		for j=1:numnp
			fprintf(fid,"%e\n",n_stress(j,i));
		end
	end
end
%%%
%%% Read FEM model data
%%%
function [numnp,node,nume,element,type,e,nu,nspc,spc,ncload,cload,vcload,ndload,dload,vdload]\
=readFEM(fid,NDIM,NN,NDOF)
%%%
%%% Node definitions:
%%%
numnp=fscanf(fid,"%d",'C');
nume=fscanf(fid,"%d",'C');
nspc=fscanf(fid,"%d",'C');
ncload=fscanf(fid,"%d",'C');
ndload=fscanf(fid,"%d",'C');
for i=1:numnp
	for j=1:NDIM
		node(i,j)=fscanf(fid,"%lf",'C');
	end
end
printf("///node :: numnp=%d\n",numnp);
for i=1:numnp
	printf("%d ",i);
	for j=1:NDIM
		printf("%e ",node(i,j));
	end
	printf("\n");
end
%%%
%%% Element definitions:
%%%
for i=1:nume
	dummy=fscanf(fid,"%d",'C');
	for j=1:NN
		element(i,j)=fscanf(fid,"%d",'C');
	end
end
printf("///element :: nume=%d\n",nume);
for i=1:nume
	printf("%d ",i);
	for j=1:NN
		printf("%d ",element(i,j) );
	end
	printf("\n");
end
%%%
%%% Material definitions:
%%%
type=fscanf(fid,"%d",'C');
e=fscanf(fid,"%lf",'C');
nu=fscanf(fid,"%lf",'C');
printf("///material :: type=%d E=%e NU=%e\n",type,e,nu);
%%%
%%% SPC definitions:
%%%
for i=1:nspc
	spc(i,1)=fscanf(fid,"%d",'C');
	spc(i,2)=fscanf(fid,"%d",'C');
end
printf("///spc :: nspc=%d\n",nspc);
for i=1:nspc
	printf("%d %d %d\n",i,spc(i,1),spc(i,2))
end
%%%
%%% load definitions:
%%%
for i=1:ncload
	cload(i,1)=fscanf(fid,"%d",'C');
	cload(i,2)=fscanf(fid,"%d",'C');
	vcload(i) =fscanf(fid,"%lf",'C');
end
printf("///cload :: ncload=%d\n",ncload);
for i=1:ncload
	printf("%d %d %d %e\n",i,cload(i,1),cload(i,2),vcload(i))
end
for i=1:ndload
	dload(i,1)=fscanf(fid,"%d",'C');
	dload(i,2)=fscanf(fid,"%d",'C');
	vdload(i)=fscanf(fid,"%lf",'C');
end
printf("///dload :: ndload=%d\n",ndload);
for i=1:ndload
	printf("%d %d %d %e\n",i,dload(i,1),dload(i,2),vdload(i))
end
end
%%%
%%%  Read FEM model data
%%%
finp=fopen('pwhCPS4.dat','r');
[numnp,node,nume,element,TYPE,E,NU,nspc,spc,ncload,cload,vcload,ndload,dload,vdload]\
=readFEM(finp,NDIM,NN,NDOF);
%%%
%%% Global stiffness (NDOF*numnp) x (NDOF*numnp)
%%%
KG=zeros(NDOF*numnp,NDOF*numnp);
%%%
%%% Loop over elements
%%%
for ie=1:nume
%%% Local element coordinate
	for i=1:NN
		for j=1:NDIM
			nodeL(i,j)=node(element(ie,i),j);
		end
	end
%%% Local elememt stiffness element
	kL=k_quad2d4(nodeL,TYPE,E,NU);
%%% Add element 1 to Global stiffness 
	for i=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,i)-1)+ii;
			for j=1:NN
				for jj=1:NDOF
					icol=NDOF*(element(ie,j)-1)+jj;
					KG(irow,icol)=KG(irow,icol)+kL(NDOF*(i-1)+ii,NDOF*(j-1)+jj);
				end
			end
		end
	end
endfor
%%%
%%% Load vector (condentrated load)
%%%
FC=zeros(NDOF*numnp,1);
for i=1:ncload
	npos=NDOF*(cload(i,1)-1)+cload(i,2);
	FC(npos)=FC(npos)+vcload(i);
end
%%%
%%% Load vector (distributed load)
%%%
FD=zeros(NDOF*numnp,1);
for i=1:ndload
	ie=dload(i,1);
	ltype=dload(i,2);
	val=vdload(i);
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
	end
	df=dload_quad2d4(nodeL,ltype,val);
	for j=1:NN
		for ii=1:NDOF
			irow=NDOF*(element(ie,j)-1)+ii;
			FD(irow)=FD(irow)+df(NDOF*(j-1)+ii);
		end
	end
end
%%%
%%% TOTAL LOAD
%%%
	F=FC+FD;
%%%
%%% constrained conditions
%%%
count=1;
for i=1:nspc
	npos=NDOF*(spc(i,1)-1)+spc(i,2);
	row(count,:)=KG(npos,:);
	KG(npos,:)=0;
	KG(:,npos)=0;
	KG(npos,npos)=1;
	F(npos)=0.0;
	count=count+1;
end
%%%
%%% computation 
%%%
printf("### Computation started\n");
U=KG\F;
printf("### Computation finished\n");
printf("//Result displacement\n")
for i=1:numnp
	printf("%d ",i);
	for j=1:NDOF
		printf("%e ",U(NDOF*(i-1)+j));
	end
	printf("\n");
end
%%% post processing :: stress and strain
%%% for nodal stress/S11,S22,S12,S33
n_stress=zeros(numnp,4);
n_count=zeros(numnp);
r1v=[-1, 1, 1,-1];
r2v=[-1,-1, 1, 1];
rr=1/sqrt(3);
nvalue=zeros(4,4);
printf("Element stress/strain @Gauss points \n")
for ie=1:nume
	for j=1:NN
		for k=1:NDIM
			nodeL(j,k)=node(element(ie,j),k);
		end
		for k=1:NDOF
			dispL(j,k)=U(NDOF*(element(ie,j)-1)+k);
		end
		nid(j)=element(ie,j);
	end
	[s e]=rcv_quad2d4(nodeL,TYPE,E,NU,dispL);
for ip=1:4
printf("%d %d S11/S22/S12/S33: %e %e %e %e\n",ie,ip,s(ip,1),s(ip,2),s(ip,3),s(ip,4));
end
for ip=1:4
printf("%d %d E11/E22/E12:     %e %e %e\n",ie,ip,e(ip,1),e(ip,2),e(ip,3));
end
	for np=1:4
		r1=r1v(np);
		r2=r2v(np);
		r1/=rr;
		r2/=rr;
		r1p=1+r1;
		r2p=1+r2;
		r1m=1-r1;
		r2m=1-r2;
		N(1)=r1m*r2m/4;
		N(2)=r1p*r2m/4;
		N(3)=r1p*r2p/4;
		N(4)=r1m*r2p/4;
		for k=1:4
			v(1)=s(1,k);
			v(2)=s(3,k);
			v(3)=s(4,k);
			v(4)=s(2,k);
			nvalue(np,k)=0.0;
			for l=1:4
				nvalue(np,k)=nvalue(np,k)+N(l)*v(l);
			end
		end
	end
%%% extrapolation to nodes
	for j=1:NN
		for k=1:4
			n_stress(nid(j),k)=n_stress(nid(j),k)+nvalue(j,k);
		end
		n_count(nid(j))=n_count(nid(j))+1;
	end
end
%%% average
for j=1:numnp
	if( n_count(j) > 0 )
		for k=1:4
			n_stress(j,k)=n_stress(j,k)/n_count(j);
		end
	end
end
printf("Nodal stress/S11 S22 S12 S33\n")
for i=1:numnp
	printf("%d %e %e %e %e\n",i,n_stress(i,1),n_stress(i,2),n_stress(i,3),n_stress(i,4));
end
%%% post processing :: reaction force
printf("Reaction force\n")
count=1;
for i=1:nspc
	f=-FD(NDOF*(spc(i,1)-1)+spc(i,2));
	for k=1:NDOF*numnp
		f=f+row(count,k)*U(k);
	end
	count=count+1;
	printf("%d %d %e\n",spc(i,1),spc(i,2),f);
end
%%%
%%% for visualization
%%%
	fid=fopen('vtk.vtk','w');
	writeVTK(fid,NDIM,NN,NDOF,numnp,nume,node,element,U,n_stress);
	fclose(fid);
%%%
%%% finished
%%%
